note
	description: "[
		Test routines specific to {JJ_BIG_NATURAL_8} numbers.

		Features from {BIG_NATURAL_TESTS}, even though fully defined in
		the {BIG_NATURAL_TESTS} ancestor class, must be redefined here
		in order to be recognized by the automatic test system.
	]"
	author: "Jimmy J. Johnson"

class
	BIG_NATURAL_8_TESTS

inherit

	BIG_NATURAL_TESTS
		redefine
				-- Initialization
			set_with_array,
				-- Status report
			is_zero,
			is_negative,
				-- Basic operations (division)
			integer_quotient,
				-- Implementation
			digit_anchor,
			number_anchor,
			testable_number_anchor
		end

feature -- Constants

	known_bits_per_word: INTEGER = 8
			-- Know number of bits in each word

	known_max_word: NATURAL_8 = 0xFF
			-- Known value of the max value representable in a word.

	known_max_half_word: NATURAL_8 = 0x0F
			-- Known value of the max value representable in a half a word

	known_max_ten_power: NATURAL_8 = 100
			-- Known value of the maximum multiple of 10 representable in a word

feature -- Basic operations (initialization tests)

	set_with_array
			-- Tests the corresponding feature from {JJ_BIG_NATURAL}.
		do
			Precursor
		end

feature -- Basic operations (Status report tests)

	is_zero
			-- Test the corresponding feature from {JJ_BIG_NATURAL}.
		do
			Precursor
		end

	is_negative
			-- Test the corresponding feature from {JJ_BIG_NATURAL}.
		do
			Precursor
		end

feature -- Test basic operations (division

	integer_quotient
			-- Test the corresponding feature from {JJ_BIG_NATURAL}.
		do
			Precursor
		end

feature -- Factory access

	new_number: JJ_BIG_NATURAL_8
			-- Factory and anchor for new big numbers.
		do
			create Result
		end

	new_number_from_integer (a_value: INTEGER): like number_anchor
			-- Create a new big number from `a_value'
		do
			create Result.from_integer (a_value)
		end

	new_number_from_value (a_value: like digit_anchor): JJ_BIG_NATURAL_8
			-- Create a new big number from `a_value'
		do
			create Result.from_value (a_value)
		end

	new_number_from_string (a_string: STRING_8): JJ_BIG_NATURAL_8
			-- Create a new big number from the contents of `a_string'.
		do
			create Result.from_string (a_string)
		end

	new_number_from_array (a_array: ARRAY [like digit_anchor]): JJ_BIG_NATURAL_8
			-- Create a new big number from contents of `a_array'.
		do
			create Result.from_array (a_array)
		end

	random: JJ_NATURAL_8_RNG
			-- A generator for 8-bit values
		once
			create Result
		end

feature {NONE} -- Anchors

	digit_anchor: NATURAL_8
			-- Anchor for declaring an entity to represent a digit.
			-- Not to be called; just used to anchor types.
			-- Declared as a feature to avoid adding an attribute.
		require else
			never_called: false
		do
			check
				do_not_call: false then
					-- Because gives no info; simply used as anchor.
			end
		end

	number_anchor: JJ_BIG_NATURAL_8
			-- Anchor when declaring numbers in descendants.
			-- Not to be called; just used to anchor types.
			-- Declared as a feature to avoid adding an attribute.
		require else
			never_called: false
		do
			check
				do_not_call: false then
					-- Because gives no info; simply used as anchor.
			end
		end

	testable_number_anchor: TESTABLE_BIG_NATURAL_8
			-- Anchor when declaring numbers for which access to
			-- all features is needed.
			-- Not to be called; just used to anchor types.
			-- Declared as a feature to avoid adding an attribute.
		require else
			never_called: false
		do
			check
				do_not_call: false then
					-- Because gives no info; simply used as anchor.
			end
		end

end
