note
	description: "[
		This class tests {JJ_BIG_NATURAL} and its four descendents:
		{JJ_BIG_NATURAL_8}, {JJ_BIG_NATURAL_16}, {JJ_BIG_NATURAL_32}, and
		{JJ_BIG_NATURAL_64}.

		In addition to running assert statements, each test feature prints
		information pertinant to that test, so that these features can be
		called from {BIG_NUMBER_DEMO} to print demonstration values.
	]"
	author: "EiffelStudio test wizard"
	date: "$Date$"
	revision: "$Revision$"
	testing: "type/manual"

deferred class
	BIG_NATURAL_TESTS

inherit

	EQA_TEST_SET
		redefine
			on_prepare
		end

feature {NONE} -- Events

	on_prepare
			-- Called after all initializations in `default_create'.
			-- Redefined to set `test_limit' for auto-testing.
		do
			test_limit := Default_test_limit
			word_limit := Default_word_limit
			digit_limit := Default_digit_limit
			power_limit := Default_power_limit
		end

feature -- Constants

	known_bits_per_word: INTEGER
			-- Know number of bits in each word
		deferred
		end

	known_max_word: like digit_anchor
			-- Known value of the max value representable in a word.
		deferred
		end

	known_max_half_word: like digit_anchor
			-- Known value of the max value representable in a half a word
		deferred
		end

	known_max_ten_power: like digit_anchor
			-- Known value of the maximum multiple of 10 representable in a word
		deferred
		end

	format_as_string: INTEGER = 0
			-- Display a {JJ_BIG_NUMBER} as a decimal number

	format_as_stored: INTEGER = 1
			-- Display a {JJ_BIG_NUMBER} using `out_as_stored'
			-- (i.e. <1,2,3,4>)

	format_as_bits: INTEGER = 2
			-- Display a {JJ_BIG_NUMBER} using `out_as_binary'
			-- (i.e. <11110011, 00011011, 00000001>)

	Default_test_limit: INTEGER = 30
			-- The default number of times to run a test.

	Default_word_limit: INTEGER = 4
			-- The default max number of words to create for tests

	Default_digit_limit: INTEGER = 10
			-- The default number of digits to create for tests

	Default_power_limit: INTEGER = 10
			-- The default maximum value allowed for exponentiation tests

feature -- Access

	test_limit: INTEGER
			-- The number of times to run a test

	word_limit: INTEGER
			-- The maximum number of words to create for tests

	digit_limit: INTEGER
			-- The maximum number of digits to create for tests

	power_limit: INTEGER
			-- The maximum exponent allowed in exponetiation tests

	target_format: INTEGER
			-- The format in which to display the target of calls is `signature'

	argument_format: INTEGER
			-- The format in which to display arguments in `signature'

	result_format: INTEGER
			-- The format in which to display results

feature -- Element change

	set_test_limit (a_limit: INTEGER)
			-- Set `test_limit' to `a_count'
		require
			count_big_enough: a_limit >= 1
		do
			test_limit := a_limit
		end

	set_word_limit (a_limit: INTEGER)
			-- Set `word_limit'
		require
			count_big_enough: a_limit >= 1
		do
			word_limit := a_limit
		end

	set_digit_limit (a_limit: INTEGER)
			-- Set `word_limit'
		require
			count_big_enough: a_limit >= 1
		do
			word_limit := a_limit
		end

	set_power_limit (a_limit: INTEGER)
			-- Set `word_limit'
		require
			count_big_enough: a_limit >= 1
		do
			power_limit := a_limit
		end

	set_target_format (a_format: INTEGER_32)
			-- Change `target_format' to `a_format'
		require
			format_big_enough: a_format >= format_as_string
			format_small_enough: a_format <= format_as_bits
		do
			target_format := a_format
		end

	set_argument_format (a_format: INTEGER_32)
			-- Change `argument_format' to `a_format'
		require
			format_big_enough: a_format >= format_as_string
			format_small_enough: a_format <= format_as_bits
		do
			argument_format := a_format
		end

	set_result_format (a_format: INTEGER_32)
			-- Change `result_format' to `a_format'
		require
			format_big_enough: a_format >= format_as_string
			format_small_enough: a_format <= format_as_bits
		do
			result_format := a_format
		end

feature -- Status report

	is_terse: BOOLEAN
			-- Should the report features print extra information?

feature -- Status setting

	set_verbose
			-- Force tests to output little or no information to `io'
		do
			is_terse := false
		end

	set_terse
			-- Force tests to output information to `io' when executed.
		do
			is_terse := true
		end

feature -- Basic operations

	run_all
			-- Demo/test all the features on the current `tester'.
		do
				-- Step into `set_with_array' and notice that the debugger
				-- shows correct argument, but on the line "d := a_array[i]
				-- the debugger shows a different value; then when `d' is
				-- extended into Current, the value is again different.
			set_with_array
				-- Feature `is_negative' fails precondition "target_closed" in
				-- this class.  This feature looks the same as `is_zero' which
				-- does not fail, but for some reason the runtime seems to thin
				-- that the target is attached to an object of the enclosing
				-- class (this one) instead of to an object of the number type.
			is_negative
		end

feature -- Test Initializations

	set_with_array
			-- Tests the corresponding feature from {JJ_BIG_NATURAL}.
		local
			i: INTEGER
			a: ARRAY [like digit_anchor]
			fn, s: STRING
			n: like number_anchor
		do
			fn := "set_with_array"
			io.put_string (divider (fn))
			from i := 1
			until i > test_limit
			loop
				a := random_array (word_limit)
				n := new_number
				s := signature (n, fn, <<a>>)
				io.put_string (s)
				n.set_with_array (a)
				io.put_string (n.out_as_stored + "%N")
				assert (s, n.out_as_stored ~ array_as_string (a))
				i := i + 1
			end
		end

feature -- Test status reports

	is_zero
			-- Tests the corresponding feature from {JJ_BIG_NATURAL}.
		local
			fn: STRING_8
			n: like number_anchor
--			mp_n: JJ_GMP_INTEGER
			i: INTEGER_32
		do
			fn := "is_zero"
			io.put_string (divider (fn))
			n := new_number
			from i := 1
			until i > test_limit
			loop
				n := new_random_near_value (n.one_word, n.four_word)
--				mp_n := new_gmp_number (n)
				compare_functions (agent n.is_zero,
--									n.out ~ mp_n.zero.out,
									n.out ~ (0).out,
									fn, {ARRAY [ANY]} <<>>)
				i := i + 1
			end
		end

	is_negative
			-- Tests the corresponding feature from {JJ_BIG_NATURAL}.
		local
			fn: STRING_8
			n: like number_anchor
--			mp_n: JJ_GMP_INTEGER
			i: INTEGER_32
		do
			fn := "is_negative"
			from i := 1
			until i > test_limit
			loop
				n := new_random_number (word_limit)
--				mp_n := new_gmp_number (n)
				compare_functions (agent n.is_negative,
--									agent mp_n.is_negative,
									True,
									fn, {ARRAY [ANY]} <<>>)
				i := i + 1
			end
		end

feature -- Test basic operations (division)

	integer_quotient
			-- Tests the corresponding feature from {JJ_BIG_NATURAL}.
		local
			fn: STRING_8
			n, n2: like number_anchor
--			mp_n, mp_n2: JJ_GMP_INTEGER
			i: INTEGER_32
		do
			fn := "integer_quotient"
			io.put_string (divider (fn))
			from i := 1
			until i > test_limit
			loop
				n := new_random_number (word_limit)
				n2 := new_random_number (word_limit)
--				mp_n := new_gmp_number (n)
--				mp_n2 := new_gmp_number (n2)
				es_random.forth
				compare_functions (agent n.integer_quotient (n2),
--									agent mp_n.integer_quotient (mp_n2),
									agent n.integer_quotient (n2),
									fn, {ARRAY [ANY]} <<n2>>)
				i := i + 1
			end
		end

feature -- Test implementation (division)

	bit_shift_left
			 -- Tests the corresponding feature from {JJ_BIG_NATURAL}.
		local
			fn: STRING_8
			n: like number_anchor
			i: INTEGER_32
		do
			fn := "bit_shift_left"
			io.put_string (divider (fn))
			from i := 0
			until i > test_limit
			loop
				n := new_random_number (word_limit)
--				n.bit_shift_left (i)
--				compare_functions (agent n.bit_shift_left (i),
--									agent mp_n.integer_remainder (mp_n2),
--									fn, {ARRAY [ANY]} <<n2>>)
				i := i + 1
			end
		end

feature {NONE} -- Implementation

	compare_functions (a_routine: ROUTINE [like number_anchor, detachable TUPLE];
						a_expected: ANY;
						a_name: STRING_8; a_args: ARRAY [ANY])
			-- Compare the result of `a_routine' from {JJ_BIG_NUMBER} to the one of:
			--     1) if `a_expected' is a {ROUTINE}, the result of executing a
			--        call to that routine, or
			--     2) if NOT a {ROUTINE}, the result of calling `out' on that argument.
			-- Return the result of the comparison, possibly printing (depending
			-- on `is_terse') the result of executing `a_routine' (and the expected
			-- value if they don't match).
			-- Arguments `a_name' and `a_args' are only used to output the `signature'
			-- of `a_function'.
		require
			target_closed: attached {like number_anchor} a_routine.target
			other_target_closed: attached {ROUTINE} a_expected as r implies
--								attached {JJ_GMP_INTEGER} r.target
								attached {like number_anchor} r.target
			arguments_closed: a_routine.open_count = 0
			other_arguments_closed: attached {ROUTINE} a_expected as r implies
										r.open_count = 0
		local
			s: STRING_8
			ans, exp: ANY	-- the answer and the expected answer
			is_ok: BOOLEAN
		do
			check attached {like number_anchor} a_routine.target as t then
					-- Print the `signature'
				s := signature (t, a_name, a_args)
				if not is_terse then
					io.put_string (s)
				end
					-- Execute `a_routine', assuming the answer is the number's value
				if attached {PROCEDURE} a_routine as p then
					p.call ([])
					ans := t
				elseif attached {FUNCTION [like number_anchor, TUPLE, ANY]} a_routine as f then
					ans := f.item ([])
				else
					check
						should_not_happen: False
							-- because `a_routine' is a {PROCEDURE} or a {FUNCTION}
					end
					ans := " should_not_happen "
				end
					-- Execute `a_other_routine_or_value' or use `out'
				if attached {PROCEDURE} a_expected as gmp_p then
					gmp_p.call ([])
					exp := gmp_p
				elseif attached {FUNCTION [JJ_GMP_INTEGER, TUPLE, ANY]} a_expected as gmp_f then
					exp := gmp_f.item ([])
				else
					exp := a_expected
				end
					-- Compare and print results
				is_ok := ans.out ~ exp.out
				if not is_terse then
					io.put_string (ans.out + "%N")
					if not is_ok then
						io.put_string ("%T ERROR -- expected   " + exp.out + "%N")
					end
				end
				assert (s, is_ok)
			end
		end

	put_line (a_string: STRING)
			-- Print a dividing line containing `a_string'
			-- (e.g.  "----------- a_string ------------"
		local
			w, c, n, i: INTEGER_32
		do
			io.put_string ("%N")
			w := 70
			c := a_string.count
			n := (w - c) // 2
			from i := 1
			until i > n
			loop
				io.put_string ("-")
				i := i + 1
			end
			io.put_string (" " + a_string + " ")
			from i := 1
			until i > n
			loop
				io.put_string ("-")
				i := i + 1
			end
			io.put_string ("%N")
		end

	divider (a_string: STRING_8): STRING_8
			-- A simple marker to output at beginning of a test feature
		do
			Result := "     -------- " + a_string + " -------- %N"
		end

	formatted_number (a_number: like number_anchor; a_format: INTEGER_32): STRING_8
			-- A string representing `a_number' displayed as `out',
			-- `out_formatted_out', `out_as_stored', or `out_as_bits',
			-- depending on `a_format'.
		do
			inspect a_format
			when Format_as_string then
				Result := a_number.out
			when Format_as_stored then
				Result := a_number.out_formatted
			when Format_as_bits then
				Result := a_number.out_as_bits
			else
				Result := " error in number format "
			end
		end

	signature (a_target: like number_anchor; a_feature: STRING_8;
				a_arguments: ARRAY [ANY]): STRING_8
			-- Create a string representing a features signature.
			-- The format for {JJ_BIG_NATURAL} numbers is set by
			-- calls to `set_format'.
		local
			i: INTEGER
			a: detachable ANY
		do
			Result := a_target.generating_type + ":  "
			Result := Result + "(" + formatted_number (a_target, target_format) + ")"
			Result := Result + "." + a_feature
			if a_arguments.count > 0 then
					-- Add agruments
				Result := Result + " ("
				from i := 1
				until i > a_arguments.count
				loop
					a := a_arguments[i]
					if attached {STRING} a as s then
						Result := Result + "%"" + s.out + "%""
					elseif attached {like number_anchor} a as n then
						Result := Result + formatted_number (n, argument_format)
					elseif attached {ARRAY [like digit_anchor]} a as otl_a then
						Result := Result + array_as_string (otl_a)
					elseif attached a then
						Result := Result + a.out
					else
						Result := Result + "Void"
					end
					if i < a_arguments.count then
						Result := Result + ", "
					end
					i := i + 1
				end
				Result := Result + ")"
			end
			Result := Result + " = "
		end

	array_as_string (a_array: ARRAY [like digit_anchor]): STRING_8
			-- Format `a_array' for output in the same format as
			-- feature `out_as_stored' from {JJ_BIG_NATURAL}.
		local
			i: INTEGER
		do
			Result := "<"
			from i := 1
			until i > a_array.count
			loop
				Result := Result + a_array[i].out
				if i < a_array.count then
					Result := Result + ","
				end
				i := i + 1
			end
			Result := Result + ">"
		end

feature {NONE} -- Implementation

	new_number: like number_anchor
			-- Factory for new big numbers.
		deferred
		end

	new_random_number (a_count: INTEGER): like number_anchor
			-- A big number with `a_count' or fewer words.
		local
			c: INTEGER_32
		do
			Result := new_number
			c := es_random.item \\ a_count + 1
			es_random.forth
			Result.set_random (c)
				-- Make some numbers negative
			if es_random.real_item < .5 then
				Result.set_is_negative (true)
			end
			es_random.forth
		end

	new_random_near_value (a_value, a_delta: like digit_anchor): like number_anchor
			-- A new one-word number that is within `a_delta' of `a_value'
		local
			v: like digit_anchor
			r: like digit_anchor
		do
			r := random.item
			random.forth
			v := (r + a_value) \\ a_delta
			Result := new_number
			Result.set_with_value (v)
				-- Make some numbers negative
			if es_random.real_item < .5 then
				Result.set_is_negative (true)
			end
		end

	new_gmp_number (a_number: NUMERIC): JJ_GMP_INTEGER
			-- Create a {JJ_GMP_INTEGER} eqivalent to `a_number'
		do
			create Result.make_string (a_number.out)
		ensure
			same_numbers: Result.out ~ a_number.out
		end

	es_random: RANDOM
			-- The EiffelBase random number generator from Eiffel Software
		once
			create Result.make
		end

	random: JJ_NATURAL_RNG [JJ_NATURAL]
			-- The RNG currently in use
		deferred
		end

	random_string (a_count: INTEGER): STRING_8
			-- A string of up to `a_count' numbers
		require
			count_large_enough: a_count >= 1
		local
			done: BOOLEAN
			n: like number_anchor
		do
			n := new_number
			random.set_range (n.one_word, n.nine_word)
				-- Now build a string of random digits
				-- Add the first number which is not a zero
			Result := random.item.out
			random.set_range (n.zero_word, n.nine_word)
			from
			until Result.count >= a_count or done
			loop
				Result := Result + random.item.out
				random.forth
					-- Finish with number possibly shorter than `a_count'
				done := random.item > n.eight_word
				random.forth
			end
		ensure
			string_long_enough: Result.count >= 1
			no_leading_zeros: not (Result.item (1) ~ '0')
		end

	random_array (a_count: INTEGER): ARRAY [like digit_anchor]
			-- An array of up to `a_count' number of digits
		require
			count_large_enough: a_count >= 1
		local
			done: BOOLEAN
			n: like number_anchor
			c: INTEGER
			i: INTEGER
		do
			c := es_random.item \\ a_count + 1
			check
				c_big_enough: c >= 1
				c_small_enough: c <= a_count
			end
			n := new_number
			create Result.make_filled (n.zero_word, 1, c)
			random.set_range (n.one_word, n.max_word)
			Result[1] := random.item
			random.set_range (n.one_word, n.max_word)
			random.forth
			from i := 1
			until i > c
			loop
				Result[i] := (random.item)
				random.forth
				i := i + 1
			end
		end

	random_array_with_range (a_count: INTEGER;
				a_lower, a_upper: like digit_anchor): ARRAY [like digit_anchor]
			-- An array of up to `a_count' number of digits, where each digit
			-- is in the range [`a_lower', `a_upper']
		require
			count_large_enough: a_count >= 1
		local
			done: BOOLEAN
			n: like number_anchor
			c: INTEGER
			i: INTEGER
		do
			c := es_random.item \\ a_count + 1
			n := new_number
			create Result.make_filled (n.zero_word, 1, c)
			random.set_range (a_lower, a_upper)
			random.forth
			from i := 1
			until i > c
			loop
				Result[i] := (random.item)
				random.forth
				i := i + 1
			end
		end

feature {NONE} -- Anchors

	digit_anchor: JJ_NATURAL
			-- Anchor for declaring an entity to representi a digit.
			-- Not to be called; just used to anchor types.
			-- Declared as a feature to avoid adding an attribute.
		require
			never_called: false
		do
			check
				do_not_call: false then
					-- Because gives no info; simply used as anchor.
			end
		end

	number_anchor: JJ_BIG_NATURAL [like digit_anchor]
			-- Anchor when declaring numbers in descendants.
			-- Not to be called; just used to anchor types.
			-- Declared as a feature to avoid adding an attribute.
		require
			never_called: false
		do
			check
				do_not_call: false then
					-- Because gives no info; simply used as anchor.
			end
		end

	gmp_anchor: JJ_GMP_INTEGER
			-- Anchor for GMP type for comparisons
		once
			check
				do_not_call: false then
			end
		end

	testable_number_anchor: TESTABLE_BIG_NATURAL [like digit_anchor]
			-- Anchor when declaring numbers for which access to
			-- all features is needed.
			-- Not to be called; just used to anchor types.
			-- Declared as a feature to avoid adding an attribute.
		require
			never_called: false
		do
			check
				do_not_call: false then
					-- Because gives no info; simply used as anchor.
			end
		end

	tuple_anchor: like number_anchor.quotient
			-- Anchor for typs involved in division.
			-- Not to be called; just used to anchor types.
			-- Declared as a feature to avoid adding an attribute.
		require
			never_called: false
		do
			check
				do_not_call: false then
					-- Because gives no info; simply used as anchor.
			end
		end

--	digit_tuple_anchor: like number_anchor.as_half_words
--			-- Anchor for typs involved in division.
--			-- Not to be called; just used to anchor types.
--			-- Declared as a feature to avoid adding an attribute.
--		require
--			never_called: false
--		do
--			check
--				do_not_call: false then
--					-- Because gives no info; simply used as anchor.
--			end
--		end

end





